/***************************************************************************
                          chart.c  -  description
                             -------------------
    begin                : Mon Sep 24 2001
    copyright            : (C) 2001 by Michael Speck
    email                : kulkanie@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lbreakout.h"
#include "config.h"
#include "chart.h"

#define CHART_FILE_NAME "lbreakout2.hscr"

enum { CHART_ENTRY_COUNT = 10 };
Chart_Entry chart[CHART_ENTRY_COUNT + 1]; /* last entry is dummy for new entry */
char chart_path[512]; /* path charts where actually loaded/created from */
int chart_loaded = 0; /* true if highscore succesfully loaded */
int chart_w, chart_h; /* size of higscore chart */
int chart_gap = 10;
int chart_level_offset = 0; /* level offset (name's left aligned, score's right aligned) */
char *cheader = "Name      Level Score";
Font *cfont = 0, *chfont = 0, *ccfont = 0; /* normal, highlight and caption font */
extern Config config;
extern Sdl sdl;

/*
====================================================================
Local
====================================================================
*/

/*
====================================================================
Reset charts
====================================================================
*/
void chart_reset()
{
    int i;
    memset( chart, 0, sizeof( Chart_Entry ) * CHART_ENTRY_COUNT );
    for ( i = 0; i < CHART_ENTRY_COUNT; i++ )
        chart_add( "............",  i + 1, 10000 * ( i + 1 ) );
    for ( i = 0; i < CHART_ENTRY_COUNT; i++ )
        chart[i].new_entry = 0;
}
/*
====================================================================
Load highscore from specified directory or create chart there if
not found. Do not use path if writing is not allowed.
ReturnValue: True if successful
====================================================================
*/
int chart_load_from_path( char *path )
{
    FILE *file = 0;
    char file_name[512];
    char buffer[128];
    int i;
    /* full file name */
    sprintf( file_name, "%s/%s", path, CHART_FILE_NAME );
    /* open file */
    file = fopen( file_name, "r" );
    if ( file ) {
        for ( i = 0; i < CHART_ENTRY_COUNT; i++ ) {
            if ( feof( file ) ) {
                fprintf( stderr, "Unexpected EOF in '%s' in entry %i... reading of highscore aborted.\n",
                         file_name, i );
                break;
            }
            fgets( buffer, 128, file ); buffer[strlen(buffer) - 1] = 0; /* remove newline */
            strcpy( chart[i].name, buffer );
            fgets( buffer, 128, file ); buffer[strlen(buffer) - 1] = 0; /* remove newline */
            chart[i].level = atoi( buffer );
            fgets( buffer, 128, file ); buffer[strlen(buffer) - 1] = 0; /* remove newline */
            chart[i].score = atoi( buffer );
        }
        fclose( file );
    }
    else {
        fprintf( stderr, "Highscore chart doesn't exist in '%s'... creating new one.\n", path );
        chart_reset();
    }
    /* test if writing is allowed without changing actual contents */
    if ( ( file = fopen( file_name, "a" ) ) == 0 ) {
        fprintf( stderr, "Write permission for '%s' denied.\n", file_name );
        return 0;
    }
    else
        fclose( file );
    strcpy( chart_path, path );
    chart_loaded = 1;
    return 1;
}

/*
====================================================================
Publics
====================================================================
*/

/*
====================================================================
Load highscore. If it doesn't exist try to create it in HI_DIR if
this fails fall back to ~/.lbreakout and create highscore there.
====================================================================
*/
void chart_load()
{
    /* load highscore */
    if ( !chart_load_from_path( HI_DIR ) ) {
        fprintf( stderr, "Unable to access highscore chart in '%s'.\n", HI_DIR );
        fprintf( stderr, "Trying to use config directory '%s'.\n", config.dir_name );
        if ( !chart_load_from_path( config.dir_name ) ) {
            fprintf( stderr, "Unable to access highscore chart in config directory... won't be "
                             "able to save any results. Sorry.\n" );
            return;
        }
    }
    printf( "Saving highscore chart in: %s\n", chart_path );
    /* load resources */
    cfont = load_fixed_font( "f_small_yellow.bmp", 32, 96, 8 );
    chfont = load_fixed_font( "f_small_white.bmp", 32, 96, 8 );
    ccfont = load_fixed_font( "f_yellow.bmp", 32, 96, 10 );
    /* compute size and position stuff of highscore */
    chart_w = text_width( ccfont, cheader );
    chart_h = cfont->height * CHART_ENTRY_COUNT + /* entries */
              chart_gap + /*gap between caption and entries */
              ccfont->height; /* caption size */
    chart_level_offset = text_width( ccfont, "name.-----" ) + text_width( ccfont, "Level" ) / 2; /* level offset centered */
}
/*
====================================================================
Delete resources
====================================================================
*/
void chart_delete()
{
    if ( cfont ) free_font( cfont ); cfont = 0;
    if ( chfont ) free_font( chfont ); chfont = 0;
    if ( ccfont ) free_font( ccfont ); ccfont = 0;
}
/*
====================================================================
Save chart in directory it was loaded from.
====================================================================
*/
void chart_save()
{
    char file_name[512];
    int i;
    FILE *file = 0;
    if ( !chart_loaded ) return;
    /* full file name */
    sprintf( file_name, "%s/%s", chart_path, CHART_FILE_NAME );
    /* open file */
    file = fopen( file_name, "w" );
    if ( !file ) {
        fprintf( stderr, "??? Highscore chart loaded properly but cannot save?\n" );
        return;
    }
    for ( i = 0; i < CHART_ENTRY_COUNT; i++ )
        fprintf( file, "%s\n%i\n%i\n", chart[i].name, chart[i].level, chart[i].score );
    fclose( file );
}
/*
====================================================================
Check if this player entered the highscore chart and update it.
====================================================================
*/
void chart_add( char *name, int level, int score )
{
    /* add new entry at blind end of chart */
    strcpy( chart[CHART_ENTRY_COUNT].name, name );
    chart[CHART_ENTRY_COUNT].level = level;
    chart[CHART_ENTRY_COUNT].score = score;
    chart[CHART_ENTRY_COUNT].new_entry = 1;
    /* sort algorithm is stable so first sort levels then score */
    chart_sort( SORT_BY_LEVEL );
    chart_sort( SORT_BY_SCORE );
}
/*
====================================================================
Sort chart with stable algorithm (bubble sort's ok) by wanted
value.
====================================================================
*/
void chart_swap( Chart_Entry *entry1, Chart_Entry *entry2 )
{
    Chart_Entry dummy;
    dummy = *entry1;
    *entry1 = *entry2;
    *entry2 = dummy;
}
void chart_sort( int type )
{
    int j;
    int changed = 0;
    /* use entry dummy as well so count is CHART_ENTRY_COUNT + 1 */
    do {
        changed = 0;
        for ( j = 0; j < CHART_ENTRY_COUNT; j++ )
            switch ( type ) {
                case SORT_BY_LEVEL:
                    if ( chart[j].level < chart[j + 1].level ) {
                        chart_swap( &chart[j], &chart[j + 1] );
                        changed = 1;
                    }
                    break;
                case SORT_BY_SCORE:
                    if ( chart[j].score < chart[j + 1].score ) {
                        chart_swap( &chart[j], &chart[j + 1] );
                        changed = 1;
                    }
                    break;
            }
    } while ( changed );
}
/*
====================================================================
Draw highscores centered in regio x,y,w,h
====================================================================
*/
void chart_show( int x, int y, int w, int h )
{
    char number_buffer[24];
    int entry_offset; /* y offset of entries */
    Font *font;
    int i;
    /* adjust x,y so it's centered */
    x += ( w - chart_w ) / 2;
    y += ( h - chart_h ) / 2;
    /* get entry offset */
    entry_offset = ccfont->height + chart_gap + y;
    /* caption */
    ccfont->align = ALIGN_X_LEFT | ALIGN_Y_TOP;
    write_text( ccfont, sdl.screen, x, y, cheader, OPAQUE );
    /* entries */
    for ( i = 0; i < CHART_ENTRY_COUNT; i++ ) {
        font = cfont;
        if ( chart[i].new_entry ) font = chfont;
        /* name */
        font->align = ALIGN_X_LEFT | ALIGN_Y_TOP;
        write_text( font, sdl.screen, x, entry_offset, chart[i].name, OPAQUE );
        /* level */
        font->align = ALIGN_X_CENTER | ALIGN_Y_TOP;
        sprintf( number_buffer, "%i", chart[i].level );
        write_text( font, sdl.screen, x + chart_level_offset, entry_offset, number_buffer, OPAQUE );
        /* score */
        font->align = ALIGN_X_RIGHT | ALIGN_Y_TOP;
        sprintf( number_buffer, "%i", chart[i].score );
        write_text( font, sdl.screen, x + chart_w, entry_offset, number_buffer, OPAQUE );
        /* change offset */
        entry_offset += font->height;
    }
    add_refresh_rect( x, y, chart_w, chart_h );
}
/*
====================================================================
Clear all new_entry flags (done before new players are added
to chart when game over).
====================================================================
*/
void chart_clear_new_entry()
{
    int i;
    chart[CHART_ENTRY_COUNT].score = chart[CHART_ENTRY_COUNT].level = 0;
    for ( i = 0; i < CHART_ENTRY_COUNT + 1; i++ ) chart[i].new_entry = 0;
}
